from .ref import CalcResult


from .component_ref import (
    ComponentEnthalpyOfFormation,
    ComponentGibbsEnergyOfFormation,
)

__all__ = [
    'ComponentEnthalpyOfFormation',
    'ComponentGibbsEnergyOfFormation',
    'CalcResult',
]
