# import libs

# SECTION: hsg properties
# NOTE: symbols
EnFo_IG_SYMBOL = 'EnFo_IG'  # enthalpy of formation symbol
Ent_STD_SYMBOL = 'Ent_STD'  # entropy of formation symbol
GiEnFo_IG_SYMBOL = 'GiEnFo_IG'  # Gibbs free energy of formation symbol
Cp_IG_SYMBOL = 'Cp_IG'  # ideal gas heat capacity symbol
VaPr_SYMBOL = 'VaPr'  # vapor pressure symbol

# NOTE: units
EnFo_IG_UNIT = 'J/mol'  # enthalpy of formation unit
Ent_STD_UNIT = 'J/mol.K'  # entropy of formation unit
GiEnFo_IG_UNIT = 'J/mol'  # Gibbs free energy of formation unit
Cp_IG_UNIT = 'J/mol.K'  # ideal gas heat capacity unit
VaPr_UNIT = 'Pa'  # vapor pressure unit
